=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-10-03 14:28:37 +0000
+++ bzrlib/branch.py	2005-10-03 14:47:12 +0000
@@ -663,6 +663,8 @@
                 continue
             if kind == 'directory':
                 inv.add(inventory.InventoryDirectory(file_id, name, parent))
+            elif kind == 'file':
+                inv.add(inventory.InventoryFile(file_id, name, parent))
             else:
                 inv.add(InventoryEntry(file_id, name, kind, parent))
         self._write_inventory(inv)

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2005-10-03 14:28:37 +0000
+++ bzrlib/inventory.py	2005-10-03 14:47:12 +0000
@@ -77,29 +77,29 @@
     'TREE_ROOT'
     >>> i.add(InventoryDirectory('123', 'src', ROOT_ID))
     InventoryDirectory('123', 'src', parent_id='TREE_ROOT')
-    >>> i.add(InventoryEntry('2323', 'hello.c', 'file', parent_id='123'))
-    InventoryEntry('2323', 'hello.c', kind='file', parent_id='123')
+    >>> i.add(InventoryFile('2323', 'hello.c', parent_id='123'))
+    InventoryFile('2323', 'hello.c', parent_id='123')
     >>> for j in i.iter_entries():
     ...   print j
     ... 
     ('src', InventoryDirectory('123', 'src', parent_id='TREE_ROOT'))
-    ('src/hello.c', InventoryEntry('2323', 'hello.c', kind='file', parent_id='123'))
-    >>> i.add(InventoryEntry('2323', 'bye.c', 'file', '123'))
+    ('src/hello.c', InventoryFile('2323', 'hello.c', parent_id='123'))
+    >>> i.add(InventoryFile('2323', 'bye.c', '123'))
     Traceback (most recent call last):
     ...
     BzrError: inventory already contains entry with id {2323}
-    >>> i.add(InventoryEntry('2324', 'bye.c', 'file', '123'))
-    InventoryEntry('2324', 'bye.c', kind='file', parent_id='123')
+    >>> i.add(InventoryFile('2324', 'bye.c', '123'))
+    InventoryFile('2324', 'bye.c', parent_id='123')
     >>> i.add(InventoryDirectory('2325', 'wibble', '123'))
     InventoryDirectory('2325', 'wibble', parent_id='123')
     >>> i.path2id('src/wibble')
     '2325'
     >>> '2325' in i
     True
-    >>> i.add(InventoryEntry('2326', 'wibble.c', 'file', '2325'))
-    InventoryEntry('2326', 'wibble.c', kind='file', parent_id='2325')
+    >>> i.add(InventoryFile('2326', 'wibble.c', '2325'))
+    InventoryFile('2326', 'wibble.c', parent_id='2325')
     >>> i['2326']
-    InventoryEntry('2326', 'wibble.c', kind='file', parent_id='2325')
+    InventoryFile('2326', 'wibble.c', parent_id='2325')
     >>> for path, entry in i.iter_entries():
     ...     print path.replace('\\\\', '/')     # for win32 os.sep
     ...     assert i.path2id(path)
@@ -235,8 +235,6 @@
 
     def kind_character(self):
         """Return a short kind indicator useful for appending to names."""
-        if self.kind == 'file':
-            return ''
         if self.kind == 'symlink':
             return ''
         raise RuntimeError('unreachable code')
@@ -253,15 +251,7 @@
 
         If no content is available, return None.
         """
-        if self.kind == 'file':
-            item.type = tarfile.REGTYPE
-            fileobj = tree.get_file(self.file_id)
-            item.size = self.text_size
-            if tree.is_executable(self.file_id):
-                item.mode = 0755
-            else:
-                item.mode = 0644
-        elif self.kind == 'symlink':
+        if self.kind == 'symlink':
             iterm.type = tarfile.SYMTYPE
             fileobj = None
             item.size = 0
@@ -283,11 +273,7 @@
 
     def _put_on_disk(self, fullpath, tree):
         """Put this entry onto disk at fullpath, from tree tree."""
-        if self.kind == 'file':
-            pumpfile(tree.get_file(self.file_id), file(fullpath, 'wb'))
-            if tree.is_executable(self.file_id):
-                os.chmod(fullpath, 0755)
-        elif self.kind == 'symlink':
+        if self.kind == 'symlink':
             try:
                 os.symlink(self.symlink_target, fullpath)
             except OSError,e:
@@ -318,26 +304,7 @@
 
     def _check(self, checker, rev_id, tree):
         """Check this inventory entry for kind specific errors."""
-        if self.kind == 'file':
-            revision = self.revision
-            t = (self.file_id, revision)
-            if t in checker.checked_texts:
-                prev_sha = checker.checked_texts[t] 
-                if prev_sha != self.text_sha1:
-                    raise BzrCheckError('mismatched sha1 on {%s} in {%s}' %
-                                        (self.file_id, rev_id))
-                else:
-                    checker.repeated_text_cnt += 1
-                    return
-            mutter('check version {%s} of {%s}', rev_id, self.file_id)
-            file_lines = tree.get_file_lines(self.file_id)
-            checker.checked_text_cnt += 1 
-            if self.text_size != sum(map(len, file_lines)):
-                raise BzrCheckError('text {%s} wrong size' % self.text_id)
-            if self.text_sha1 != sha_strings(file_lines):
-                raise BzrCheckError('text {%s} wrong sha1' % self.text_id)
-            checker.checked_texts[t] = self.text_sha1
-        elif self.kind == 'symlink':
+        if self.kind == 'symlink':
             if self.text_sha1 != None or self.text_size != None or self.text_id != None:
                 raise BzrCheckError('symlink {%s} has text in revision {%s}'
                         % (self.file_id, rev_id))
@@ -352,10 +319,6 @@
     def copy(self):
         other = InventoryEntry(self.file_id, self.name, self.kind,
                                self.parent_id)
-        other.executable = self.executable
-        other.text_id = self.text_id
-        other.text_sha1 = self.text_sha1
-        other.text_size = self.text_size
         other.symlink_target = self.symlink_target
         other.revision = self.revision
         # note that children are *not* copied; they're pulled across when
@@ -549,6 +512,81 @@
                    self.parent_id))
 
 
+class InventoryFile(InventoryEntry):
+    """A file in an inventory."""
+
+    def _check(self, checker, rev_id, tree):
+        """See InventoryEntry._check"""
+        revision = self.revision
+        t = (self.file_id, revision)
+        if t in checker.checked_texts:
+            prev_sha = checker.checked_texts[t] 
+            if prev_sha != self.text_sha1:
+                raise BzrCheckError('mismatched sha1 on {%s} in {%s}' %
+                                    (self.file_id, rev_id))
+            else:
+                checker.repeated_text_cnt += 1
+                return
+        mutter('check version {%s} of {%s}', rev_id, self.file_id)
+        file_lines = tree.get_file_lines(self.file_id)
+        checker.checked_text_cnt += 1 
+        if self.text_size != sum(map(len, file_lines)):
+            raise BzrCheckError('text {%s} wrong size' % self.text_id)
+        if self.text_sha1 != sha_strings(file_lines):
+            raise BzrCheckError('text {%s} wrong sha1' % self.text_id)
+        checker.checked_texts[t] = self.text_sha1
+
+    def copy(self):
+        other = InventoryFile(self.file_id, self.name, self.parent_id)
+        other.executable = self.executable
+        other.text_id = self.text_id
+        other.text_sha1 = self.text_sha1
+        other.text_size = self.text_size
+        other.revision = self.revision
+        return other
+
+    def __init__(self, file_id, name, parent_id):
+        super(InventoryFile, self).__init__(file_id, name, 'file',
+                                                 parent_id)
+
+    def kind_character(self):
+        """See InventoryEntry.kind_character."""
+        return ''
+
+    def __new__(cls, file_id, name, parent_id):
+        """Only present until the InventoryEntry.__new__ can go away."""
+        result = object.__new__(InventoryFile, file_id, name, 'directory',
+                                parent_id)
+        # type.__call__ is strange, it doesn't __init__ when the returned type
+        # differs
+        #result.__init__(file_id, name, 'directory', parent_id)
+        return result
+
+    def _put_in_tar(self, item, tree):
+        """See InventoryEntry._put_in_tar."""
+        item.type = tarfile.REGTYPE
+        fileobj = tree.get_file(self.file_id)
+        item.size = self.text_size
+        if tree.is_executable(self.file_id):
+            item.mode = 0755
+        else:
+            item.mode = 0644
+        return fileobj
+
+    def _put_on_disk(self, fullpath, tree):
+        """See InventoryEntry._put_on_disk."""
+        pumpfile(tree.get_file(self.file_id), file(fullpath, 'wb'))
+        if tree.is_executable(self.file_id):
+            os.chmod(fullpath, 0755)
+
+    def __repr__(self):
+        return ("%s(%r, %r, parent_id=%r)"
+                % (self.__class__.__name__,
+                   self.file_id,
+                   self.name,
+                   self.parent_id))
+
+
 class Inventory(object):
     """Inventory of versioned files in a tree.
 
@@ -768,6 +806,8 @@
 
         if kind == 'directory':
             ie = InventoryDirectory(file_id, parts[-1], parent_id)
+        elif kind == 'file':
+            ie = InventoryFile(file_id, parts[-1], parent_id)
         else:
             ie = InventoryEntry(file_id, parts[-1],
                                 kind=kind, parent_id=parent_id)

=== modified file 'bzrlib/selftest/testinv.py'
--- bzrlib/selftest/testinv.py	2005-10-03 14:28:37 +0000
+++ bzrlib/selftest/testinv.py	2005-10-03 14:47:12 +0000
@@ -68,7 +68,7 @@
 class TestInventoryEntry(TestCaseInTempDir):
 
     def test_file_kind_character(self):
-        file = InventoryEntry('123', 'hello.c', 'file', ROOT_ID)
+        file = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
         self.assertEqual(file.kind_character(), '')
 
     def test_dir_kind_character(self):
@@ -91,9 +91,9 @@
         self.assertEqual((False, False), right.detect_changes(left))
 
     def test_file_detect_changes(self):
-        left = InventoryEntry('123', 'hello.c', 'file', ROOT_ID)
+        left = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
         left.text_sha1 = 123
-        right = InventoryEntry('123', 'hello.c', 'file', ROOT_ID)
+        right = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
         right.text_sha1 = 123
         self.assertEqual((False, False), left.detect_changes(right))
         self.assertEqual((False, False), right.detect_changes(left))
@@ -119,7 +119,7 @@
         self.assertEqual((True, False), right.detect_changes(left))
 
     def test_file_has_text(self):
-        file = InventoryEntry('123', 'hello.c', 'file', ROOT_ID)
+        file = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
         self.failUnless(file.has_text())
 
     def test_directory_has_text(self):

=== modified file 'bzrlib/xml4.py'
--- bzrlib/xml4.py	2005-10-03 14:28:37 +0000
+++ bzrlib/xml4.py	2005-10-03 14:47:12 +0000
@@ -100,20 +100,23 @@
             ie = inventory.InventoryDirectory(elt.get('file_id'),
                                               elt.get('name'),
                                               parent_id)
+        elif kind == 'file':
+            ie = inventory.InventoryFile(elt.get('file_id'),
+                                         elt.get('name'),
+                                         parent_id)
+            ie.text_id = elt.get('text_id')
+            ie.text_sha1 = elt.get('text_sha1')
+            v = elt.get('text_size')
+            ie.text_size = v and int(v)
         else:
             ie = InventoryEntry(elt.get('file_id'),
                                 elt.get('name'),
                                 elt.get('kind'),
                                 parent_id)
-            ie.text_id = elt.get('text_id')
-            ie.text_sha1 = elt.get('text_sha1')
             ie.symlink_target = elt.get('symlink_target')
 
         ## mutter("read inventoryentry: %r" % (elt.attrib))
 
-        v = elt.get('text_size')
-        ie.text_size = v and int(v)
-
         return ie
 
 

=== modified file 'bzrlib/xml5.py'
--- bzrlib/xml5.py	2005-10-03 14:28:37 +0000
+++ bzrlib/xml5.py	2005-10-03 14:47:12 +0000
@@ -130,17 +130,21 @@
             ie = inventory.InventoryDirectory(elt.get('file_id'),
                                               elt.get('name'),
                                               parent_id)
+        elif kind == 'file':
+            ie = inventory.InventoryFile(elt.get('file_id'),
+                                         elt.get('name'),
+                                         parent_id)
+            ie.text_sha1 = elt.get('text_sha1')
+            if elt.get('executable') == 'yes':
+                ie.executable = True
+            v = elt.get('text_size')
+            ie.text_size = v and int(v)
         else:
             ie = InventoryEntry(elt.get('file_id'),
                                 elt.get('name'),
                                 kind,
                                 parent_id)
-            ie.text_sha1 = elt.get('text_sha1')
             ie.symlink_target = elt.get('symlink_target')
-            if elt.get('executable') == 'yes':
-                ie.executable = True
-            v = elt.get('text_size')
-            ie.text_size = v and int(v)
         ie.revision = elt.get('revision')
 
         return ie

